{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "# Using Aviary and OpenAeroStruct Together\n",
    "\n",
    "This is an example of an external subsystem using the [OpenAeroStruct (OAS)](https://github.com/mdolab/OpenAeroStruct) structural analysis system to perform aerostructural optimization of a typical large single aisle transport aircraft wing.\n",
    "The subsystem is based on the [OpenAeroStruct aerostructural optimization with wingbox](https://mdolab-openaerostruct.readthedocs-hosted.com/en/latest/aerostructural_wingbox_walkthrough.html) example problem.\n",
    "\n",
    "This example performs a sub-optimization for minimum wing weight that is then used by Aviary.\n",
    "Another use case would be to perform a structural analysis only.\n",
    "Structural design variables would be passed to the subsystem from Aviary and wing weight and a constraint or constraints representing the structural analysis would be passed back to Aviary for use by the top level optimization.\n",
    "\n",
    "## Motivation\n",
    "\n",
    "There may be a need for a higher fidelity tool to compute wing weight instead of relying on the empirical methods in core Aviary.\n",
    "A structural analysis external tool is usually used because of an unusual aircraft configuration that may not be predicted my Aviary empirical weight estimation methods, but in this example case it is simply a demonstration of an external capability to compute wing weight.\n",
    "\n",
    "## External Dependencies\n",
    "\n",
    "The user must install [OpenAeroStruct](https://github.com/mdolab/OpenAeroStruct) into their Python environment using the command 'pip install openaerostruct'.\n",
    "The user must also install the [ambiance](https://ambiance.readthedocs.io/en/latest/) package using the command 'pip install ambiance'.\n",
    "\n",
    "## Subsystem Details\n",
    "\n",
    "There are two parts to building an external subsystem -- the analysis tool and the Aviary external subsystem builder for that tool.\n",
    "The analysis tool takes inputs and parameters from Aviary and return outputs that Aviary can use to override existing variables.\n",
    "The subsystem builder uses the Aviary external subsystem builder template to connect the analysis tool to Aviary as either a pre-mission, mission or post-mission subsystem.\n",
    "\n",
    "For this case, the analysis tool will compute a wing weight in the pre-mission portion of the Aviary analysis and return its value to Aviary to override the empirical wing weight value.\n",
    "Fuel weight is passed in from Aviary as the only input currently, but other inputs may also be passed in through the subsystem builder, [OAS_wing_weight_builder](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/OAS_wing_weight_builder.py), by the `promotes_inputs` parameter.\n",
    "Other Aviary variables can also be added as additional inputs based on user needs.\n",
    "\n",
    "```{note}\n",
    "Some modifications of the [OAS_wing_weight_analysis](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/OAS_wing_weight_analysis.py) code will be necessary to add new inputs not already defined.\n",
    "```\n",
    "\n",
    "Here is the builder object for the OAS wing weight analysis example:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# %load ../../examples/external_subsystems/OAS_weight/OAS_wing_weight_builder.py\n",
    "\"\"\"\n",
    "Builder for an OpenAeroStruct component that computes a new wing mass.\n",
    "\n",
    "\"\"\"\n",
    "import openmdao.api as om\n",
    "import aviary.api as av\n",
    "from aviary.examples.external_subsystems.OAS_weight.OAS_wing_weight_analysis import OAStructures\n",
    "\n",
    "\n",
    "class OASWingWeightBuilder(av.SubsystemBuilderBase):\n",
    "    def __init__(self, name='wing_weight'):\n",
    "        super().__init__(name)\n",
    "\n",
    "    def build_pre_mission(self, aviary_inputs):\n",
    "        '''\n",
    "        Build an OpenMDAO system for the pre-mission computations of the subsystem.\n",
    "\n",
    "        Returns\n",
    "        -------\n",
    "        pre_mission_sys : openmdao.core.System\n",
    "            An OpenMDAO system containing all computations that need to happen in\n",
    "            the pre-mission part of the Aviary problem. This\n",
    "            includes sizing, design, and other non-mission parameters.\n",
    "        '''\n",
    "\n",
    "        wing_group = om.Group()\n",
    "        wing_group.add_subsystem(\"aerostructures\",\n",
    "                                 OAStructures(\n",
    "                                     symmetry=True,\n",
    "                                     wing_weight_ratio=1.0,\n",
    "                                     S_ref_type='projected',\n",
    "                                     n_point_masses=1,\n",
    "                                     num_twist_cp=4,\n",
    "                                     num_box_cp=51),\n",
    "                                 promotes_inputs=[\n",
    "                                     ('fuel', av.Mission.Design.FUEL_MASS),\n",
    "                                 ],\n",
    "                                 promotes_outputs=[('wing_weight', av.Aircraft.Wing.MASS)])\n",
    "\n",
    "        return wing_group\n"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Analysis Model Details\n",
    "\n",
    "This analysis is based on the Aviary benchmark [aircraft_for_bench_FwFm](https://github.com/OpenMDAO/Aviary/blob/main/aviary/models/test_aircraft/aircraft_for_bench_FwFm.csv) input data representing a typical large single aisle class transport aircraft.\n",
    "The analysis code [OAS_wing_weight_analysis](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/OAS_wing_weight_analysis.py) contains the `OAStructures` class which performs a structural analysis of the wing. The image below shows a simplified XDSM diagram of the pre-mission data flow in this example.\n",
    "\n",
    "![OAS XDSM](images/OAS_xdsm.PNG)\n",
    "\n",
    "We'll now discuss this code in more detail.\n",
    "\n",
    "First, we create a mesh that defines the shape of the wing based on the span, the location of the wing break typical of a transport aircraft, the dihedral and the wing root, wing break, and wing tip chord lengths.\n",
    "The twist of the wing is defined along the span at a set of control points and must be present as it is used in the optimization problem.\n",
    "\n",
    "We then use this mesh to define a simplified finite element model of the wing structural box.\n",
    "We also define the airfoil shapes as an input to OpenAeroStruct for a given wing thickness to chord ratio (t/c) to represent the wing box thickness.\n",
    "We then set initial values for the wing skin thickness and spar thickness are set, along with material properties and stress allowables for a metal material, typically aluminum.\n",
    "OpenAeroStruct will then calculate aeroelastic loads for a 2.5g maneuver condition and apply those loads to the finite element model of the wing structure.\n",
    "\n",
    "Results of the structural optimization determine the optimum wing skin thickness, spar cap thickness, wing twist, wing t/c and maneuver angle of attack that satisfies strength constraints while minimizing wing weight.\n",
    "The 'OAStructures' class returns the optimized wing mass and the fuel mass burned but currently only the wing mass is used to override the Aviary variable 'Aircraft.Wing.MASS'.\n",
    "\n",
    "The [OAS_wing_weight_analysis](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/OAS_wing_weight_analysis.py) code may be executed in Python to test the OpenAeroStruct analysis outside of the Aviary subsystem interface.\n",
    "Default values for each of the inputs and options are included at the bottom of the analysis code file.\n",
    "This can be a useful test to demonstrate that the OpenAeroStruct analysis model has been properly defined and the model returns reasonable results.\n",
    "\n",
    "Once the user is satisfied that the results are acceptable, the analysis tool can then be added as an external subsystem and tested in a mission.\n",
    "\n",
    "## Subsystem variables\n",
    "\n",
    "A variety of parameters may be defined for an OpenAeroStruct model.\n",
    "These allow the user to control how the aerodynamic and finite element meshes are subdivided, give details about the aerodynamic solution and provide structural material properties and structural scaling factors.\n",
    "The input variables passed in from Aviary may include the fuel mass, reserve fuel mass, airfoil description, engine mass and its location, lift and drag coefficients and the cruise conditions of Mach, altitude, thrust-specific fuel consumption (SFC) and range.\n",
    "\n",
    "This is a list of the available options defined for the structural analysis:\n",
    "\n",
    "- symmetry\n",
    "- chord_cos_spacing\n",
    "- span_cos_spacing\n",
    "- num_box_cp\n",
    "- num_twist_cp\n",
    "- S_ref_type\n",
    "- fem_model_type\n",
    "- with_viscous\n",
    "- with_wave\n",
    "- k_lam\n",
    "- c_max_t\n",
    "- E\n",
    "- G\n",
    "- yield\n",
    "- mrho\n",
    "- strength_factor_for_upper_skin\n",
    "- wing_weight_ratio\n",
    "- exact_failure_constraint\n",
    "- struct_weight_relief\n",
    "- distributed_fuel_weight\n",
    "- n_point_masses\n",
    "- fuel_density\n",
    "\n",
    "This is a list of the inputs defined for the structural analysis:\n",
    "\n",
    "- box_upper_x\n",
    "- box_lower_x\n",
    "- box_upper_y\n",
    "- box_lower_y\n",
    "- twist_cp\n",
    "- spar_thickness_cp\n",
    "- skin_thickness_cp\n",
    "- t_over_c_cp\n",
    "- airfoil_t_over_c\n",
    "- fuel\n",
    "- fuel_reserve\n",
    "- CL0\n",
    "- CD0\n",
    "- cruise_Mach\n",
    "- cruise_altitude\n",
    "- cruise_range\n",
    "- cruise_SFC\n",
    "- engine_mass\n",
    "- engine_location\n",
    "\n",
    "The 2 outputs from the analysis tool are:\n",
    "\n",
    "- wing_weight\n",
    "- fuel_burn\n",
    "\n",
    "See [OAS_wing_weight_analysis](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/OAS_wing_weight_analysis.py) and the [OpenAeroStruct aerostructural optimization with wingbox](https://mdolab-openaerostruct.readthedocs-hosted.com/en/latest/aerostructural_wingbox_walkthrough.html) documentation for descriptions of these variables."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "remove-cell"
    ]
   },
   "outputs": [],
   "source": [
    "# Testing Cell\n",
    "# ********NOTE********\n",
    "# check variables here?"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Test Case\n",
    "\n",
    "A simple Aviary mission is defined to test the inclusion of the OpenAeroStruct wing weight subsystem during the pre-mission phase.\n",
    "The test mission is defined in [run_simple_OAS_mission](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS/run_simple_OAS_mission.py) and is a mission based on input data read from the benchmark data file [aircraft_for_bench_FwFm](https://github.com/OpenMDAO/Aviary/blob/main/aviary/models/test_aircraft/aircraft_for_bench_FwFm.csv).\n",
    "\n",
    "The OpenAeroStruct subsystem is used to compute an optimum wing mass which will override the Aviary computed wing mass value.\n",
    "The value of the Aviary variable `Aircraft.Wing.MASS` is printed at the conclusion of the mission to verify that the wing weight from the subsystem is overriding the Aviary computed wing weight.\n",
    "\n",
    "A variable in the [run_simple_OAS_mission](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS/run_simple_OAS_mission.py) file named `use_OAS` may be set by the user to `True` or `False` to run the simple mission with or without the OpenAeroStruct subsystem included.\n",
    "This will allow the mission to be flown either using the Aviary empirical wing weight estimation (`use_OAS=False`) or with the OpenAeroStruct subsystem (`use_OAS=True`)."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "remove-cell"
    ]
   },
   "outputs": [],
   "source": [
    "# Testing Cell\n",
    "import aviary.api as av\n",
    "from aviary.examples.external_subsystems.OAS_weight.run_simple_OAS_mission import use_OAS\n",
    "av.Aircraft.Wing.MASS;"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Wing weight optimization of this type usually does not have knowledge of non-optimum wing mass values such as leading and training edge structure, actuators, stiffeners, etc.\n",
    "The optimum wing mass computed by the `OAStructures` class can be scaled using the option `wing_weight_ratio` to better match either the Aviary empirical wing weight value or a known fly-away weight estimate for your wing model.\n",
    "One method to determine the wing_weight_ratio would be to run the mission to calculate the Aviary empirical wing weight and then run [OAS_wing_weight_analysis](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/OAS_wing_weight_analysis.py) by itself using its default input values and compare wing weights.  The `wing_weight_ratio` value may then be set to calibrate the OpenAeroStruct wing weight to the expected fly-away weight.\n",
    "\n",
    "This calibration step has already been performed for this model, so the user may run the simple mission with or without the OpenAeroStruct subsystem active and compare the results.\n",
    "\n",
    "## Example Run Script\n",
    "\n",
    "Here is the full run script used to run the simple mission with the OpenAeroStruct subsystem active.\n",
    "This run script is also available in the [run_simple_OAS_mission file.](https://github.com/OpenMDAO/Aviary/blob/main/aviary/examples/external_subsystems/OAS_weight/run_simple_OAS_mission.py)\n",
    "\n",
    "\n",
    "```{note}\n",
    "We do not actually perform the optimization below.\n",
    "Instead, we define and set up the model and the call to run the optimization is commented you.\n",
    "You can uncomment this and run the code block to perform a full optimization.\n",
    "```"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# %load ../../examples/external_subsystems/OAS_weight/run_simple_OAS_mission.py\n",
    "'''\n",
    "This is a simple test mission to demonstrate the inclusion of a\n",
    "pre-mission user defined external subsystem.  The simple mission\n",
    "is based on input data read from the benchmark data file bench_4.csv,\n",
    "which represents a single-aisle commercial transport aircraft.  The\n",
    "OpenAeroStruct (OAS) subsystem is used to compute an optimum wing\n",
    "mass which will override the Aviary computed wing mass value.\n",
    "\n",
    "The flag 'use_OAS' is set to 'True' to include the OAS subsystem in\n",
    "the mission, or set to 'False' to run the mission without the\n",
    "subsystem so that wing mass values between the 2 methods may be\n",
    "compared.\n",
    "\n",
    "'''\n",
    "\n",
    "import numpy as np\n",
    "import openmdao.api as om\n",
    "import aviary.api as av\n",
    "from aviary.examples.external_subsystems.OAS_weight.OAS_wing_weight_builder import OASWingWeightBuilder\n",
    "\n",
    "# flag to turn on/off OpenAeroStruct subsystem for comparison testing\n",
    "use_OAS = True\n",
    "\n",
    "wing_weight_builder = OASWingWeightBuilder()\n",
    "\n",
    "# Load the phase_info and other common setup tasks\n",
    "phase_info = {\n",
    "    'climb_1': {\n",
    "        'subsystem_options': {'core_aerodynamics': {'method': 'computed'}},\n",
    "        'user_options': {\n",
    "            'optimize_mach': False,\n",
    "            'optimize_altitude': False,\n",
    "            'polynomial_control_order': 1,\n",
    "            'num_segments': 5,\n",
    "            'order': 3,\n",
    "            'solve_for_distance': False,\n",
    "            'initial_mach': (0.2, 'unitless'),\n",
    "            'final_mach': (0.72, 'unitless'),\n",
    "            'mach_bounds': ((0.18, 0.74), 'unitless'),\n",
    "            'initial_altitude': (0.0, 'ft'),\n",
    "            'final_altitude': (32000.0, 'ft'),\n",
    "            'altitude_bounds': ((0.0, 34000.0), 'ft'),\n",
    "            'throttle_enforcement': 'path_constraint',\n",
    "            'fix_initial': True,\n",
    "            'constrain_final': False,\n",
    "            'fix_duration': False,\n",
    "            'initial_bounds': ((0.0, 0.0), 'min'),\n",
    "            'duration_bounds': ((64.0, 192.0), 'min'),\n",
    "        },\n",
    "        'initial_guesses': {'time': ([0, 128], 'min')},\n",
    "    },\n",
    "    'climb_2': {\n",
    "        'subsystem_options': {'core_aerodynamics': {'method': 'computed'}},\n",
    "        'user_options': {\n",
    "            'optimize_mach': False,\n",
    "            'optimize_altitude': False,\n",
    "            'polynomial_control_order': 1,\n",
    "            'num_segments': 5,\n",
    "            'order': 3,\n",
    "            'solve_for_distance': False,\n",
    "            'initial_mach': (0.72, 'unitless'),\n",
    "            'final_mach': (0.72, 'unitless'),\n",
    "            'mach_bounds': ((0.7, 0.74), 'unitless'),\n",
    "            'initial_altitude': (32000.0, 'ft'),\n",
    "            'final_altitude': (34000.0, 'ft'),\n",
    "            'altitude_bounds': ((23000.0, 38000.0), 'ft'),\n",
    "            'throttle_enforcement': 'boundary_constraint',\n",
    "            'fix_initial': False,\n",
    "            'constrain_final': False,\n",
    "            'fix_duration': False,\n",
    "            'initial_bounds': ((64.0, 192.0), 'min'),\n",
    "            'duration_bounds': ((56.5, 169.5), 'min'),\n",
    "        },\n",
    "        'initial_guesses': {'time': ([128, 113], 'min')},\n",
    "    },\n",
    "    'descent_1': {\n",
    "        'subsystem_options': {'core_aerodynamics': {'method': 'computed'}},\n",
    "        'user_options': {\n",
    "            'optimize_mach': False,\n",
    "            'optimize_altitude': False,\n",
    "            'polynomial_control_order': 1,\n",
    "            'num_segments': 5,\n",
    "            'order': 3,\n",
    "            'solve_for_distance': False,\n",
    "            'initial_mach': (0.72, 'unitless'),\n",
    "            'final_mach': (0.36, 'unitless'),\n",
    "            'mach_bounds': ((0.34, 0.74), 'unitless'),\n",
    "            'initial_altitude': (34000.0, 'ft'),\n",
    "            'final_altitude': (500.0, 'ft'),\n",
    "            'altitude_bounds': ((0.0, 38000.0), 'ft'),\n",
    "            'throttle_enforcement': 'path_constraint',\n",
    "            'fix_initial': False,\n",
    "            'constrain_final': True,\n",
    "            'fix_duration': False,\n",
    "            'initial_bounds': ((120.5, 361.5), 'min'),\n",
    "            'duration_bounds': ((29.0, 87.0), 'min'),\n",
    "        },\n",
    "        'initial_guesses': {'time': ([241, 58], 'min')},\n",
    "    },\n",
    "    'post_mission': {\n",
    "        'include_landing': False,\n",
    "        'constrain_range': True,\n",
    "        'target_range': (1800., 'nmi'),\n",
    "    },\n",
    "}\n",
    "\n",
    "phase_info['pre_mission'] = {'include_takeoff': False, 'optimize_mass': True}\n",
    "if use_OAS:\n",
    "    phase_info['pre_mission']['external_subsystems'] = [wing_weight_builder]\n",
    "\n",
    "aircraft_definition_file = 'models/test_aircraft/aircraft_for_bench_FwFm.csv'\n",
    "mission_method = 'simple'\n",
    "mass_method = 'FLOPS'\n",
    "make_plots = False\n",
    "max_iter = 100\n",
    "optimizer = 'IPOPT'\n",
    "\n",
    "prob = av.AviaryProblem()\n",
    "\n",
    "prob.load_inputs(aircraft_definition_file, phase_info)\n",
    "prob.check_and_preprocess_inputs()\n",
    "prob.add_pre_mission_systems()\n",
    "prob.add_phases()\n",
    "prob.add_post_mission_systems()\n",
    "prob.link_phases()\n",
    "prob.add_driver(optimizer=optimizer, max_iter=max_iter)\n",
    "prob.add_design_variables()\n",
    "prob.add_objective()\n",
    "prob.setup()\n",
    "\n",
    "if use_OAS:\n",
    "    OAS_sys = 'pre_mission.wing_weight.aerostructures.'\n",
    "    prob.set_val(OAS_sys + 'box_upper_x', np.array([0.1, 0.11, 0.12, 0.13, 0.14, 0.15, 0.16, 0.17, 0.18, 0.19, 0.2, 0.21, 0.22, 0.23, 0.24, 0.25, 0.26, 0.27, 0.28, 0.29, 0.3, 0.31, 0.32,\n",
    "                 0.33, 0.34, 0.35, 0.36, 0.37, 0.38, 0.39, 0.4, 0.41, 0.42, 0.43, 0.44, 0.45, 0.46, 0.47, 0.48, 0.49, 0.5, 0.51, 0.52, 0.53, 0.54, 0.55, 0.56, 0.57, 0.58, 0.59, 0.6]), units='unitless')\n",
    "    prob.set_val(OAS_sys + 'box_lower_x', np.array([0.1, 0.11, 0.12, 0.13, 0.14, 0.15, 0.16, 0.17, 0.18, 0.19, 0.2, 0.21, 0.22, 0.23, 0.24, 0.25, 0.26, 0.27, 0.28, 0.29, 0.3, 0.31, 0.32,\n",
    "                 0.33, 0.34, 0.35, 0.36, 0.37, 0.38, 0.39, 0.4, 0.41, 0.42, 0.43, 0.44, 0.45, 0.46, 0.47, 0.48, 0.49, 0.5, 0.51, 0.52, 0.53, 0.54, 0.55, 0.56, 0.57, 0.58, 0.59, 0.6]), units='unitless')\n",
    "    prob.set_val(OAS_sys + 'box_upper_y', np.array([0.0447,  0.046,  0.0472,  0.0484,  0.0495,  0.0505,  0.0514,  0.0523,  0.0531,  0.0538, 0.0545,  0.0551,  0.0557, 0.0563,  0.0568, 0.0573,  0.0577,  0.0581,  0.0585,  0.0588,  0.0591,  0.0593,  0.0595,  0.0597,\n",
    "                 0.0599,  0.06,    0.0601,  0.0602,  0.0602,  0.0602,  0.0602,  0.0602,  0.0601,  0.06,    0.0599,  0.0598,  0.0596,  0.0594,  0.0592,  0.0589,  0.0586,  0.0583,  0.058,   0.0576,  0.0572,  0.0568,  0.0563,  0.0558,  0.0553,  0.0547,  0.0541]), units='unitless')\n",
    "    prob.set_val(OAS_sys + 'box_lower_y', np.array([-0.0447, -0.046, -0.0473, -0.0485, -0.0496, -0.0506, -0.0515, -0.0524, -0.0532, -0.054, -0.0547, -0.0554, -0.056, -0.0565, -0.057, -0.0575, -0.0579, -0.0583, -0.0586, -0.0589, -0.0592, -0.0594, -0.0595, -0.0596, -\n",
    "                 0.0597, -0.0598, -0.0598, -0.0598, -0.0598, -0.0597, -0.0596, -0.0594, -0.0592, -0.0589, -0.0586, -0.0582, -0.0578, -0.0573, -0.0567, -0.0561, -0.0554, -0.0546, -0.0538, -0.0529, -0.0519, -0.0509, -0.0497, -0.0485, -0.0472, -0.0458, -0.0444]), units='unitless')\n",
    "    prob.set_val(OAS_sys + 'twist_cp', np.array([-6., -6., -4., 0.]), units='deg')\n",
    "    prob.set_val(OAS_sys + 'spar_thickness_cp',\n",
    "                 np.array([0.004, 0.005, 0.008, 0.01]), units='m')\n",
    "    prob.set_val(OAS_sys + 'skin_thickness_cp',\n",
    "                 np.array([0.005, 0.01, 0.015, 0.025]), units='m')\n",
    "    prob.set_val(OAS_sys + 't_over_c_cp',\n",
    "                 np.array([0.08, 0.08, 0.10, 0.08]), units='unitless')\n",
    "    prob.set_val(OAS_sys + 'airfoil_t_over_c', 0.12, units='unitless')\n",
    "    prob.set_val(OAS_sys + 'fuel', 40044.0, units='lbm')\n",
    "    prob.set_val(OAS_sys + 'fuel_reserve', 3000.0, units='lbm')\n",
    "    prob.set_val(OAS_sys + 'CD0', 0.0078, units='unitless')\n",
    "    prob.set_val(OAS_sys + 'cruise_Mach', 0.785, units='unitless')\n",
    "    prob.set_val(OAS_sys + 'cruise_altitude', 11303.682962301647, units='m')\n",
    "    prob.set_val(OAS_sys + 'cruise_range', 3500, units='nmi')\n",
    "    prob.set_val(OAS_sys + 'cruise_SFC', 0.53 / 3600, units='1/s')\n",
    "    prob.set_val(OAS_sys + 'engine_mass', 7400, units='lbm')\n",
    "    prob.set_val(OAS_sys + 'engine_location', np.array([25, -10.0, 0.0]), units='m')\n",
    "\n",
    "prob.set_initial_guesses()\n",
    "# prob.run_aviary_problem('dymos_solution.db', make_plots=False)\n",
    "# print('wing mass = ', prob.model.get_val(av.Aircraft.Wing.MASS, units='lbm'))\n"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {
    "tags": [
     "remove-cell"
    ]
   },
   "outputs": [],
   "source": [
    "# Testing Cell\n",
    "from aviary.interface.utils.check_phase_info import check_phase_info, HEIGHT_ENERGY\n",
    "check_phase_info(phase_info, HEIGHT_ENERGY);"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "base",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.10.13"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 2
}
